package pl.edu.pg.eti.kask.rpg.cmd;

import com.fasterxml.jackson.databind.ObjectWriter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;
import pl.edu.pg.eti.kask.rpg.ansi.function.ImageToAnsiArtFunction;
import pl.edu.pg.eti.kask.rpg.character.controller.api.CharacterController;
import pl.edu.pg.eti.kask.rpg.character.dto.PutCharacterRequest;

import java.util.NoSuchElementException;
import java.util.Scanner;
import java.util.UUID;

/**
 * Component for interaction with user using command line. Components annotated with {@link @Component} implementing
 * {@link CommandLineRunner} are executed automatically..
 */
@Component
public class ApplicationCommand implements CommandLineRunner {

    /**
     * Service for managing characters. Should be injected automatically.
     */
    private final CharacterController characterController;

    /**
     * Writer for JSON.
     */
    private final ObjectWriter writer;

    /**
     * Converts image to ANSI art.
     */
    private final ImageToAnsiArtFunction imageToAnsiArt;

    /**
     * @param characterController controller for accessing characters
     * @param writer              writer for JSON
     * @param imageToAnsiArt      converts image to ANSI art
     */
    @Autowired
    public ApplicationCommand(
            CharacterController characterController,
            ObjectWriter writer,
            ImageToAnsiArtFunction imageToAnsiArt
    ) {
        this.characterController = characterController;
        this.writer = writer;
        this.imageToAnsiArt = imageToAnsiArt;
    }

    @Override
    public void run(String... args) throws Exception {
        Scanner scanner = new Scanner(System.in);
        String command;
        main_loop:
        while (true) {
            command = scanner.next();
            switch (command) {
                case "get_characters" -> {
                    System.out.println(writer.writeValueAsString(characterController.getCharacters()));
                }
                case "get_user_characters" -> {
                    UUID uuid = UUID.fromString(scanner.next());
                    try {
                        System.out.println(writer.writeValueAsString(characterController.getUserCharacters(uuid)));
                    } catch (NoSuchElementException ex) {
                        System.out.println("NOT_FOUND");
                    }
                }
                case "get_profession_characters" -> {
                    UUID uuid = UUID.fromString(scanner.next());
                    try {
                        System.out.println(writer.writeValueAsString(characterController.getProfessionCharacters(uuid)));
                    } catch (NoSuchElementException ex) {
                        System.out.println("NOT_FOUND");
                    }
                }
                case "get_character" -> {
                    UUID uuid = UUID.fromString(scanner.next());
                    try {
                        System.out.println(writer.writeValueAsString(characterController.getCharacter(uuid)));
                    } catch (NoSuchElementException ex) {
                        System.out.println("NOT_FOUND");
                    }
                }
                case "get_character_portrait" -> {
                    UUID uuid = UUID.fromString(scanner.next());
                    try {
                        System.out.println(imageToAnsiArt.apply(characterController.getCharacterPortrait(uuid), 40));
                    } catch (NoSuchElementException ex) {
                        System.out.println("NOT_FOUND");
                    }
                }
                case "delete_character" -> {
                    try {
                        UUID uuid = UUID.fromString(scanner.next());
                        characterController.deleteCharacter(uuid);
                    } catch (NoSuchElementException ex) {
                        System.out.println("Not Found");
                    }
                }
                case "put_character" -> {
                    UUID uuid = UUID.fromString(scanner.next());
                    PutCharacterRequest request = PutCharacterRequest.builder()
                            .name(scanner.next())
                            .profession(UUID.fromString(scanner.next()))
                            .build();
                    try {
                        characterController.putCharacter(uuid, request);
                    } catch (IllegalArgumentException ex) {
                        System.out.println("Bad Request");
                    }
                }
                case "quit" -> {
                    break main_loop;
                }
            }
        }
    }


}
